searchData={"items":[{"type":"behaviour","title":"ElixirMake.Precompiler","doc":"The behaviour for precompiler modules.","ref":"ElixirMake.Precompiler.html"},{"type":"callback","title":"ElixirMake.Precompiler.all_supported_targets/1","doc":"This callback should return a list of triplets (\"arch-os-abi\") for all supported targets\nof the given operation.\n\nFor the `:compile` operation, `all_supported_targets` should return a list of targets that\nthe current host is capable of (cross-)compiling to.\n\nFor the `:fetch` operation, `all_supported_targets` should return the full list of targets.\n\nFor example, GitHub Actions provides Linux, macOS and Windows CI hosts, when `operation` is\n`:compile`, the precompiler might return `[\"x86_64-linux-gnu\"]` if it is running in the Linux\nCI environment, while returning `[\"x86_64-apple-darwin\", \"aarch64-apple-darwin\"]` on macOS,\nor `[\"amd64-windows\", \"x86-windows\"]` on Windows platform.\n\nWhen `operation` is `:fetch`, the precompiler should return the full list. The full list for\nthe above example should be:\n\n    [\n      \"x86_64-linux-gnu\",\n      \"x86_64-apple-darwin\",\n      \"aarch64-apple-darwin\",\n      \"amd64-windows\",\n      \"x86-windows\"\n    ]\n\nThis allows the precompiler to do the compilation work in multilple hosts and gather all the\nartefacts later with `mix elixir_make.checksum --all`.","ref":"ElixirMake.Precompiler.html#c:all_supported_targets/1"},{"type":"callback","title":"ElixirMake.Precompiler.build_native/1","doc":"This callback will be invoked when the user executes the `mix compile`\n(or `mix compile.elixir_make`) command.\n\nThe precompiler should then compile the NIF library \"natively\". Note that\nit is possible for the precompiler module to pick up other environment variables\nlike `TARGET_ARCH=aarch64` and adjust compile arguments correspondingly.","ref":"ElixirMake.Precompiler.html#c:build_native/1"},{"type":"callback","title":"ElixirMake.Precompiler.current_target/0","doc":"This callback should return the target triplet for current node.","ref":"ElixirMake.Precompiler.html#c:current_target/0"},{"type":"function","title":"ElixirMake.Precompiler.mix_compile/1","doc":"Invoke the regular Mix toolchain compilation.","ref":"ElixirMake.Precompiler.html#mix_compile/1"},{"type":"callback","title":"ElixirMake.Precompiler.post_precompile/0","doc":"Optional post actions to run after all precompilation tasks are done.\n\nIt will only be called at the end of the `mix elixir_make.precompile` command.\nFor example, actions can be archiving all precompiled artefacts and uploading\nthe archive file to an object storage server.","ref":"ElixirMake.Precompiler.html#c:post_precompile/0"},{"type":"callback","title":"ElixirMake.Precompiler.post_precompile_target/1","doc":"Optional post actions to run after each precompilation target is archived.\n\nIt will be called when a target is precompiled and archived successfully.\nFor example, actions can be deleting all target-specific files.","ref":"ElixirMake.Precompiler.html#c:post_precompile_target/1"},{"type":"callback","title":"ElixirMake.Precompiler.precompile/2","doc":"This callback should precompile the library to the given target(s).\n\nReturns `:ok` if the requested target has successfully compiled.","ref":"ElixirMake.Precompiler.html#c:precompile/2"},{"type":"callback","title":"ElixirMake.Precompiler.unavailable_target/1","doc":"Optional recover actions when the current target is unavailable.\n\nThere are two reasons that the current target might be unavailable:\nwhen the library only has precompiled binaries for some platforms,\nand it either\n\n- needs to be compiled on other platforms.\n\n  The callback should return `:compile` for this case.\n\n- is intended to function as `noop` on other platforms.\n\n  The callback should return `:ignore` for this case.\n\nDefaults to `:compile` if this callback is not implemented.","ref":"ElixirMake.Precompiler.html#c:unavailable_target/1"},{"type":"type","title":"ElixirMake.Precompiler.target/0","doc":"Target triplet.","ref":"ElixirMake.Precompiler.html#t:target/0"},{"type":"task","title":"mix compile.elixir_make","doc":"Runs `make` in the current project.\n\nThis task runs `make` in the current project; any output coming from `make` is\nprinted in real-time on stdout.","ref":"Mix.Tasks.Compile.ElixirMake.html"},{"type":"task","title":"Configuration - mix compile.elixir_make","doc":"This compiler can be configured through the return value of the `project/0`\nfunction in `mix.exs`; for example:\n\n    def project() do\n      [app: :myapp,\n       make_executable: \"make\",\n       make_makefile: \"Othermakefile\",\n       compilers: [:elixir_make] ++ Mix.compilers,\n       deps: deps()]\n    end\n\nThe following options are available:\n\n  * `:make_executable` - (binary or `:default`) it's the executable to use as the\n    `make` program. If not provided or if `:default`, it defaults to `\"nmake\"`\n    on Windows, `\"gmake\"` on FreeBSD, OpenBSD and NetBSD, and `\"make\"` on everything\n    else. You can, for example, customize which executable to use on a\n    specific OS and use `:default` for every other OS. If the `MAKE`\n    environment variable is present, that is used as the value of this option.\n\n  * `:make_makefile` - (binary or `:default`) it's the Makefile to\n    use. Defaults to `\"Makefile\"` for Unix systems and `\"Makefile.win\"` for\n    Windows systems if not provided or if `:default`.\n\n  * `:make_targets` - (list of binaries) it's the list of Make targets that\n    should be run. Defaults to `[]`, meaning `make` will run the first target.\n\n  * `:make_clean` - (list of binaries) it's a list of Make targets to be run\n    when `mix clean` is run. It's only run if a non-`nil` value for\n    `:make_clean` is provided. Defaults to `nil`.\n\n  * `:make_cwd` - (binary) it's the directory where `make` will be run,\n    relative to the root of the project.\n\n  * `:make_env` - (map of binary to binary) it's a map of extra environment\n    variables to be passed to `make`. You can also pass a function in here in\n    case `make_env` needs access to things that are not available during project\n    setup; the function should return a map of binary to binary. Many default\n    environment variables are set, see section below\n\n  * `:make_error_message` - (binary or `:default`) it's a custom error message\n    that can be used to give instructions as of how to fix the error (e.g., it\n    can be used to suggest installing `gcc` if you're compiling a C\n    dependency).\n\n  * `:make_args` - (list of binaries) it's a list of extra arguments to be passed.\n\nThe following options configure precompilation:\n\n  * `:make_precompiler` - a two-element tuple with the precompiled type\n    and module to use. The precompile type is either `:nif` or `:port`\n    and then the precompilation module. If the type is a `:nif`, it looks\n    for a DDL or a shared object as precompilation target given by\n    `:make_precompiler_filename` and the current NIF version is part of\n    the precompiled archive. If `:port`, it looks for an executable with\n    `:make_precompiler_filename`.\n\n  * `:make_precompiler_url` - the download URL template. Defaults to none.\n    Required when `make_precompiler` is set.\n\n  * `:make_precompiler_filename` - the filename of the compiled artefact\n    without its extension. Defaults to the app name.\n\n  * `:make_force_build` - if build should be forced even if precompiled artefacts\n    are available. Defaults to true if the app has a `-dev` version flag.\n\nSee [the Precompilation guide](PRECOMPILATION_GUIDE.md) for more information.","ref":"Mix.Tasks.Compile.ElixirMake.html#module-configuration"},{"type":"task","title":"Default environment variables - mix compile.elixir_make","doc":"There are also several default environment variables set:\n\n  * `MIX_TARGET`\n  * `MIX_ENV`\n  * `MIX_BUILD_PATH` - same as `Mix.Project.build_path/0`\n  * `MIX_APP_PATH` - same as `Mix.Project.app_path/0`\n  * `MIX_COMPILE_PATH` - same as `Mix.Project.compile_path/0`\n  * `MIX_CONSOLIDATION_PATH` - same as `Mix.Project.consolidation_path/0`\n  * `MIX_DEPS_PATH` - same as `Mix.Project.deps_path/0`\n  * `MIX_MANIFEST_PATH` - same as `Mix.Project.manifest_path/0`\n  * `ERL_EI_LIBDIR`\n  * `ERL_EI_INCLUDE_DIR`\n  * `ERTS_INCLUDE_DIR`\n  * `ERL_INTERFACE_LIB_DIR`\n  * `ERL_INTERFACE_INCLUDE_DIR`\n\nThese may also be overwritten with the `make_env` option.","ref":"Mix.Tasks.Compile.ElixirMake.html#module-default-environment-variables"},{"type":"task","title":"Compilation artifacts and working with priv directories - mix compile.elixir_make","doc":"Generally speaking, compilation artifacts are written to the `priv`\ndirectory, as that the only directory, besides `ebin`, which are\navailable to Erlang/OTP applications.\n\nHowever, note that Mix projects supports the `:build_embedded`\nconfiguration, which controls if assets in the `_build` directory\nare symlinked (when `false`, the default) or copied (`true`).\nIn order to support both options for `:build_embedded`, it is\nimportant to follow the given guidelines:\n\n  * The \"priv\" directory must not exist in the source code\n  * The Makefile should copy any artifact to `$MIX_APP_PATH/priv`\n    or, even better, to `$MIX_APP_PATH/priv/$MIX_TARGET`\n  * If there are static assets, the Makefile should copy them over\n    from a directory at the project root (not named \"priv\")","ref":"Mix.Tasks.Compile.ElixirMake.html#module-compilation-artifacts-and-working-with-priv-directories"},{"type":"task","title":"mix elixir_make.checksum","doc":"A task responsible for downloading the precompiled NIFs for a given module.\n\nThis task must only be used by package creators who want to ship the\nprecompiled NIFs. The goal is to download the precompiled packages and\ngenerate a checksum to check-in alongside the project in the the Hex repository.\nThis is done by passing the `--all` flag.\n\nYou can also use the `--only-local` flag to download only the precompiled\npackage for use during development.\n\nYou can use the `--ignore-unavailable` flag to ignore any NIFs that are not available.\nThis is useful when you are developing a new NIF that does not support all platforms.\n\nThis task also accept the `--print` flag to print the checksums.","ref":"Mix.Tasks.ElixirMake.Checksum.html"},{"type":"task","title":"mix elixir_make.precompile","doc":"Precompiles the given project for all targets.\n\nThis task must only be used by package creators who want to ship the\nprecompiled NIFs. This task is often used on CI to precompile\nfor different targets.\n\nThis is only supported if `:make_precompiler` is specified\nin your project configuration.","ref":"Mix.Tasks.ElixirMake.Precompile.html"},{"type":"extras","title":"Precompilation guide","doc":"# Precompilation guide\n\nThis guide has two sections, the first one is intended for precompiler module developers. It covers a minimal example of creating a precompiler module. The second section is intended for library developers who want their library to be able to use precompiled artefacts in a simple way.\n\n- [Library Developer](#library-developer)\n- [Precompiler Module Developer](#precompiler-module-developer)","ref":"precompilation_guide.html"},{"type":"extras","title":"Library Developer - Precompilation guide","doc":"This guide assumes you have already added `elixir_make` to your library and you have written a `Makefile` that compiles the native code in your project. Once your native code compile and works as expected, you are now ready to precompile it.\n\nA full demo project is available on [cocoa-xu/cc_precompiler_example](https://github.com/cocoa-xu/cc_precompiler_example).\n\n#","ref":"precompilation_guide.html#library-developer"},{"type":"extras","title":"Setup mix.exs - Precompilation guide","doc":"To use a precompiler module such as the `CCPrecompiler` example above, we first add the precompiler (`:cc_precompiler` here) and `:elixir_make` to `deps`.\n\n```elixir\ndef deps do\n[\n    # ...\n    {:elixir_make, \"~> 0.6\", runtime: false},\n    {:cc_precompiler, \"~> 0.1.0\", runtime: false, github: \"cocoa-xu/cc_precompiler\"}\n    # ...\n]\nend\n```\n\nThen add `:elixir_make` to the `compilers` list, and set the type (`:nif` or `:port`) and `CCPrecompile` as the value for `:make_precompiler`.\n\n```elixir\n@version \"0.1.0\"\ndef project do\n  [\n    # ...\n    compilers: [:elixir_make] ++ Mix.compilers(),\n    # elixir_make specific config\n    # required\n    make_precompiler: {:nif, CCPrecompiler},\n    make_precompiler_url: \"https://github.com/cocoa-xu/cc_precompiler_example/releases/download/v#{@version}/@{artefact_filename}\",\n\n    # optional\n    make_precompiler_filename: \"nif\",\n    make_precompiler_priv_paths: [\"nif.*\"],\n    make_precompiler_nif_versions: [\n      versions: [\"2.14\", \"2.15\", \"2.16\"]\n    ]\n    # ...\n  ]\nend\n```\n\nAnother required field is `make_precompiler_url`. It is a URL template to the artefact file.\n\n`@{artefact_filename}` in the URL template string will be replaced by corresponding artefact filenames when fetching them. For example, `cc_precompiler_example-nif-2.16-x86_64-linux-gnu-0.1.0.tar.gz`.\n\n#### `make_precompiler_filename` (optional config key)\n\nThe first optional config key for elixir_make is `make_precompiler_filename`. If the name (file extension does not count) of the shared library is different from your app's name, then `make_precompiler_filename` should be set. For example, if the app name is `\"cc_precompiler_example\"` while the name shared library is `\"nif.so\"` (or `\"nif.dll\"` on windows), then `make_precompiler_filename` should be set as `\"nif\"`.\n\n#### `make_precompiler_priv_paths` (optional config key)\nThe second optional config key is `make_precompiler_priv_paths`. For example, say the `priv` directory is organised as follows in Linux, macOS and Windows respectively,\n\n```\n# Linux\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.so\n│   ├── libpriv2.so\n│   └── libpriv3.so\n└── nif.so\n\n# macOS\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.dylib\n│   ├── libpriv2.dylib\n│   └── libpriv3.dylib\n└── nif.so\n\n# Windows\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.dll\n│   ├── libpriv2.dll\n│   └── libpriv3.dll\n└── nif.dll\n```\n\nBy default, everything in `priv` will be included in the precompiled tar file. However, files in `assets` can be very large or platform-independent, therefore, we would like to only include the `nif.so` (`nif.dll`) file and everything in the `lib` directory in the precompiled tar file to reduce the footprint. In this case, we can set `make_precompiler_priv_paths` to `[\"nif.so\", \"nif.dll\", \"lib\"]`.\n\nOf course, wildcards (`?`, `**`, `*`) are supported when specifiying files. For example, `[\"nif.*\", \"lib/*.so\", \"lib/*.dll\", \"lib/*.dylib\"]` will include `nif.so` (Linux/macOS) or `nif.dll` (Windows), and `.so` or `.dll` files in the `lib` directory. \n\nDirectory structures and symbolic links are preserved.\n\n#### `make_precompiler_nif_versions` (optional config key)\n\nThe third optional config key is `make_precompiler_nif_versions`, which configures `elixir_make` on how to compile and reuse precompiled binaries.\n\nThe default value for `make_precompiler_nif_versions` is \n\n```elixir\n[versions: [\"#{:erlang.system_info(:nif_version)}\"]]\n```\n\nThere're two sub-keys for `make_precompiler_nif_versions`:\n\n- `versions` \n- `fallback_version`\n\n##### `versions` sub-key\n\nThe `versions` sub-key is either a list of NIF versions of a function that returns a list of NIF versions that the precompiled artefacts are available for:\n\n```elixir\nmake_precompiler_nif_versions: [\n  versions: [\"2.15\", \"2.16\"]\n]\n```\n\nThe above example tells `:elixir_make` that all targets have precompiled artefacts for NIF version `2.15` and `2.16`.\n\nFor some platforms maybe we only have precompiled artefacts after a certain NIF version, say for x86_64 Windows we have precompiled artefacts available when NIF version >= `2.16` while other platforms have precompiled artefacts available from NIF version >= `2.15`.\n\nIn such case we can inform `:elixir_make` that Windows targets don't have precompiled artefacts available except for NIF version `2.16` by passing a function to the `availability` sub-key. This function should accept two arguments, `target` and `nif_version`, and returns a boolean value indicating whether the precompiled artefacts for the target and NIF version are available.\n\n```elixir\nmake_precompiler_nif_versions: [\n  versions: fn opts ->\n    target = opts.target\n    if String.contains?(target, \"windows\") do\n      [\"2.16\"]\n    else\n      [\"2.15\", \"2.16\"]\n    end\n  end\n]\n```\n\nThe default behaviour is to use the exact NIF version that is available to the current target. If one is not available, it may fallback (see `fallback_version` next) to the highest matching major version prior to the current version. For example:\n\n- if the current host is using Erlang/OTP 23 (NIF version `2.15`), `elixir_make` will use the precompiled artefacts for NIF version `2.15`; \n- if the current host is using Erlang/OTP 24 or 25 (NIF version `2.16`), `elixir_make` will use the precompiled artefacts for NIF version `2.16`; \n- if the current host is using Erlang/OTP 26 or newer (NIF version `2.17`), `elixir_make` will fallback to the precompiled artefacts for NIF version `2.16`;\n\nIf the current host is using Erlang/OTP with a new major Erlang NIF version (NIF version `3.0`) or anything earlier than the precompiled versions (`2.14`), `elixir_make` will compile from scratch.\n\n##### `fallback_version` sub-key\n\nThe behaviour when `elixir_make` cannot find the exact NIF version of the precompiled binary can be customized by setting the `fallback_version` sub-key.\n\nThe value of the `fallback_version` sub-key should be a function that accepts one argument, `opts`, which is a map that include one key `target`. The `target` is the target triplet (or other naming format, defined by the precompiler of your choice).\n\nThe `fallback_version` function should return the NIF version that is available and should be chosen for the current target.\n\n### (Optional) Customise Precompilation Targets\n\nTo override the default configuration, please set the `cc_precompile` key in `project`. For example,\n\n```elixir\n\ndef project do\n[ \n  # ...\n  cc_precompile: [\n    # optional config that provides a map of available compilers\n    # on different systems\n    compilers: %{\n      # key (`:os.type()`)\n      #   this allows us to provide different available targets \n      #   on different systems\n      # value is a map that describes which compilers are available\n      #\n      # key == {:unix, :linux} => when compiling on Linux\n      {:unix, :linux} => %{\n        # key (target triplet) => `riscv64-linux-gnu`\n        # value => `PREFIX`\n        #   - for strings, the string will be used as the prefix of\n        #         the C and C++ compiler respectively, i.e.,\n        #         CC=`#{prefix}gcc`\n        #         CXX=`#{prefix}g++`\n        \"riscv64-linux-gnu\" => \"riscv64-linux-gnu-\",\n        # key (target triplet) => `armv7l-linux-gnueabihf`\n        # value => `{CC, CXX}`\n        #   - for 2-tuples, the elements are the executable name of\n        #         the C and C++ compiler respectively\n        \"armv7l-linux-gnueabihf\" => {\n          \"arm-linux-gnueabihf-gcc\",\n          \"arm-linux-gnueabihf-g++\"\n        },\n        # key (target triplet) => `armv7l-linux-gnueabihf`\n        # value => `{CC_EXECUTABLE, CXX_EXECUTABLE, CC_TEMPLATE, CXX_TEMPLATE}`\n        #\n        # - for 4-tuples, the first two elements are the same as in\n        #       2-tuple, the third and fourth elements are the template\n        #       string for CC and CPP/CXX. for example,\n        #       \n        #       the last entry below shows the example of using zig as the\n        #       crosscompiler for `aarch64-linux-musl`, \n        #       the \"CC\" will be\n        #           \"zig cc -target aarch64-linux-musl\", \n        #       and \"CXX\" and \"CPP\" will be\n        #           \"zig c++ -target aarch64-linux-musl\"\n        \"aarch64-linux-musl\" => {\n          \"zig\", \n          \"zig\", \n          \"<% cc %> cc -target aarch64-linux-musl\", \n          \"<% cxx %> c++ -target aarch64-linux-musl\"\n        }\n      },\n      # key == {:unix, :darwin} => when compiling on macOS\n      {:unix, :darwin} => %{\n        # key (target triplet) => `aarch64-apple-darwin`\n        # value => `{CC, CXX}`\n        \"aarch64-apple-darwin\" => {\n          \"gcc -arch arm64\", \"g++ -arch arm64\"\n        },\n        # key (target triplet) => `aarch64-linux-musl`\n        # value => `{CC_EXECUTABLE, CXX_EXECUTABLE, CC_TEMPLATE, CXX_TEMPLATE}`\n        \"aarch64-linux-musl\" => {\n          \"zig\",\n          \"zig\",\n          \"<% cc %> cc -target aarch64-linux-musl\",\n          \"<% cxx %> c++ -target aarch64-linux-musl\"\n        }\n      }\n    }\n  ]\n]\n```\n\n### (Optional) Test the NIF code locally\n\nTo test the NIF code locally, you can either set `force_build` to `true` or append `\"-dev\"` to your NIF library's version string.\n\n```elixir\n@version \"0.1.0-dev\"\n\ndef project do\n  [\n    # either append `\"-dev\"` to your NIF library's version string\n    version: @version,\n    # or set force_build to true\n    make_force_build: true,\n    # ...\n  ]\nend\n```\n\nDoing so will ask `elixir_make` to only compile for the current host instead of building for all available targets.\n\n```shell\n$ mix compile\ncc -shared -std=c11 -O3 -fPIC -I\"/usr/local/lib/erlang/erts-13.0.3/include\" -undefined dynamic_lookup -flat_namespace -undefined suppress \"/Users/cocoa/git/cc_precompiler_example/c_src/cc_precompiler_example.c\" -o \"/Users/cocoa/Git/cc_precompiler_example/_build/dev/lib/cc_precompiler_example/priv/nif.so\"\n$ mix test\nmake: Nothing to be done for `build'.\nGenerated cc_precompiler_example app\n.\n\nFinished in 0.00 seconds (0.00s async, 0.00s sync)\n1 test, 0 failures\n\nRandomized with seed 102464\n```\n\n#","ref":"precompilation_guide.html#setup-mix-exs"},{"type":"extras","title":"Precompile for available targets - Precompilation guide","doc":"It's possible to either setup a CI task to do the precompilation job or precompile on a local machine and upload the precompiled artefacts.\n\nTo precompile for all targets on a local machine:\n\n```shell\nMIX_ENV=prod mix elixir_make.precompile\n```\n\nEnvironment variable `ELIXIR_MAKE_CACHE_DIR` can be used to set the cache dir for the precompiled artefacts, for instance, to output precompiled artefacts in the cache directory of the current working directory, `export ELIXIR_MAKE_CACHE_DIR=\"$(pwd)/cache\"`.\n\nTo setup a CI task such as GitHub Actions, the following workflow file can be used for reference:\n\n```yml\nname: precompile\n\non:\n  push:\n    tags:\n      - 'v*'\n\njobs:\n  linux:\n    runs-on: ubuntu-latest\n    env:\n      MIX_ENV: \"prod\"\n    steps:\n      - uses: actions/checkout@v3\n      - uses: erlef/setup-beam@v1\n        with:\n          otp-version: \"25.0.2\"\n          elixir-version: \"1.13.4\"\n      - name: Install system dependecies\n        run: |\n          sudo apt-get update\n          sudo apt-get install -y build-essential automake autoconf pkg-config bc m4 unzip zip \\\n            gcc-aarch64-linux-gnu g++-aarch64-linux-gnu \\\n            gcc-riscv64-linux-gnu g++-riscv64-linux-gnu\n      - name: Mix Test\n        run: |\n          mix deps.get\n          MIX_ENV=test mix test\n      - name: Create precompiled library\n        run: |\n          export ELIXIR_MAKE_CACHE_DIR=$(pwd)/cache\n          mkdir -p \"${ELIXIR_MAKE_CACHE_DIR}\"\n          mix elixir_make.precompile\n      - uses: softprops/action-gh-release@v1\n        if: startsWith(github.ref, 'refs/tags/')\n        with:\n          files: |\n            cache/*.tar.gz\n\n  macos:\n    runs-on: macos-11\n    env:\n      MIX_ENV: \"prod\"\n    steps:\n      - uses: actions/checkout@v3\n      - name: Install erlang and elixir\n        run: |\n          brew install erlang elixir\n          mix local.hex --force\n          mix local.rebar --force\n      - name: Mix Test\n        run: |\n          mix deps.get\n          MIX_ENV=test mix test\n      - name: Create precompiled library\n        run: |\n          export ELIXIR_MAKE_CACHE_DIR=$(pwd)/cache\n          mkdir -p \"${ELIXIR_MAKE_CACHE_DIR}\"\n          mix elixir_make.precompile\n      - uses: softprops/action-gh-release@v1\n        if: startsWith(github.ref, 'refs/tags/')\n        with:\n          files: |\n            cache/*.tar.gz\n```\n\n#","ref":"precompilation_guide.html#precompile-for-available-targets"},{"type":"extras","title":"Generate checksum file - Precompilation guide","doc":"After CI has finished, you can fetch the precompiled binaries from GitHub.\n\n```shell\n$ MIX_ENV=prod mix elixir_make.checksum --all --ignore-unavailable\n```\n\nMeanwhile, a checksum file will be generated. In this example, the checksum file will be named as `checksum-cc_precompiler_example.exs` in current working directory.\n\nThis checksum file is extremely important in the scenario where you need to release a Hex package using precompiled NIFs. It's **MANDATORY** to include this file in your Hex package (by updating the `files` field in the `mix.exs`). Otherwise your package **won't work**.\n\n```elixir\ndefp package do\n  [\n    files: [\n      \"lib\",\n      \"checksum.exs\",\n      \"mix.exs\",\n      # ...\n    ],\n    # ...\n  ]\nend\n```\n\nHowever, there is no need to track the checksum file in your version control system (git or other), so consider adding it to your `.gitignore`.\n\n### (Optional) Test fetched artefacts can work locally\n\n```shell\n# delete previously built binaries so that\n# elixir_make will try to restore the NIF library\n# from the downloaded tarball file\n$ rm -rf _build/prod/lib/cc_precompiler_example\n# set to prod env and test everything\n$ MIX_ENV=prod mix test\n==> castore\nCompiling 1 file (.ex)\nGenerated castore app\n==> elixir_make\nCompiling 5 files (.ex)\nGenerated elixir_make app\n==> cc_precompiler\nCompiling 1 file (.ex)\nGenerated cc_precompiler app\n\n20:47:42.262 [debug] Restore NIF for current node from: /Users/cocoa/Library/Caches/cc_precompiler_example-nif-2.16-aarch64-apple-darwin-0.1.0.tar.gz\n==> cc_precompiler_example\nCompiling 1 file (.ex)\nGenerated cc_precompiler_example app\n.\n\nFinished in 0.01 seconds (0.00s async, 0.01s sync)\n1 test, 0 failures\n\nRandomized with seed 539590\n```\n\n#","ref":"precompilation_guide.html#generate-checksum-file"},{"type":"extras","title":"Recommended flow - Precompilation guide","doc":"To recap, the suggested flow is the following:\n\n1. Choose an appropriate precompiler for your NIF library and set all necessary options in the `mix.exs`.\n2. (Optional) Test if your NIF library compiles locally.\n\n  ```shell\n  mix compile\n  mix test\n  ```\n\n3. (Optional) Test if your NIF library can precompile to all specified targets locally.\n\n  ```shell\n  MIX_ENV=prod mix elixir_make.precompile\n  ```\n\n4. Precompile your library on CI or locally.\n\n  ```shell\n  # locally\n  MIX_ENV=prod mix elixir_make.precompile\n  # CI\n  # please see the docs above\n  ```\n\n5. Fetch precompiled binaries from GitHub.\n\n  ```shell\n  # only fetch artefact for current host\n  MIX_ENV=prod mix elixir_make.checksum --only-local --print\n  # fetch all\n  MIX_ENV=prod mix elixir_make.checksum --all --print\n  # to fetch all available artefacts at the moment\n  MIX_ENV=prod mix elixir_make.checksum --all --print --ignore-unavailable\n  ```\n\n6. (Optional) Test if the downloaded artefacts works as expected.\n\n  ```shell\n  rm -rf _build/prod/lib/NIF_LIBRARY_NAME\n  MIX_ENV=prod mix test\n  ```\n\n6. Update Hex package to include the checksum file.\n\n7. Release the package to Hex.pm (make sure your release includes the correct files).","ref":"precompilation_guide.html#recommended-flow"},{"type":"extras","title":"Precompiler Module Developer - Precompilation guide","doc":"In this section, I'll walk you through creating a simple precompiler that utilises existing crosscompilers in the system.\n\n#","ref":"precompilation_guide.html#precompiler-module-developer"},{"type":"extras","title":"Create a new precompiler module - Precompilation guide","doc":"We start by creating a new elixir library, say `cc_precompiler`.\n\n```shell\n$ mix new cc_precompiler\n* creating README.md\n* creating .formatter.exs\n* creating .gitignore\n* creating mix.exs\n* creating lib\n* creating lib/cc_precompiler.ex\n* creating test\n* creating test/test_helper.exs\n* creating test/cc_precompiler_test.exs\n\nYour Mix project was created successfully.\nYou can use \"mix\" to compile it, test it, and more:\n\n    cd cc_precompiler\n    mix test\n\nRun \"mix help\" for more commands.\n```\n\nThen in the `mix.exs` file, we add `:elixir_make` to `deps`.\n\n```elixir\ndefp deps do\n  [{:elixir_make, \"~> 0.6\", runtime: false}]\nend\n```\n\n#","ref":"precompilation_guide.html#create-a-new-precompiler-module"},{"type":"extras","title":"Write the CC Precompiler - Precompilation guide","doc":"To create a precompiler module that is compatible with `elixir_make`, the module (`lib/cc_precompiler.ex`) need to implement a few callbacks defined in the `ElixirMake.Precompiler` beheviour.\n\nThe full project of `cc_precompiler` is available on [cocoa-xu/cc_precompiler](https://github.com/cocoa-xu/cc_precompiler).\n\n```elixir\ndefmodule CCPrecompiler do\n  @moduledoc \"\"\"\n  Precompile with existing crosscompiler in the system.\n  \"\"\"\n\n  require Logger\n  @behaviour ElixirMake.Precompiler\n\n  # This is the default configuration for this demo precompiler module\n  # for linux systems, it will detect for the following targets\n  #   - x86_64-linux-gnu\n  #   - i686-linux-gnu\n  #   - aarch64-linux-gnu\n  #   - armv7l-linux-gnueabihf\n  #   - riscv64-linux-gnu\n  #   - powerpc64le-linux-gnu\n  #   - s390x-linux-gnu\n  # by trying to find the corresponding executable, i.e.,\n  #   - x86_64-linux-gnu-gcc\n  #   - i686-linux-gnu-gcc\n  #   - aarch64-linux-gnu-gcc\n  #   - arm-linux-gnueabihf-gcc\n  #   - riscv64-linux-gnu-gcc\n  #   - powerpc64le-linux-gnu-gcc\n  #   - s390x-linux-gnu-gcc\n  # (this module will only try to find the CC executable, a step further\n  # will be trying to compile a simple C/C++ program using them)\n  @default_compilers %{\n    {:unix, :linux} => %{\n      \"x86_64-linux-gnu\" => \"x86_64-linux-gnu-\",\n      \"i686-linux-gnu\" => \"i686-linux-gnu-\",\n      \"aarch64-linux-gnu\" => \"aarch64-linux-gnu-\",\n      \"armv7l-linux-gnueabihf\" => \"arm-linux-gnueabihf-\",\n      \"riscv64-linux-gnu\" => \"riscv64-linux-gnu-\",\n      \"powerpc64le-linux-gnu\" => \"powerpc64le-linux-gnu-\",\n      \"s390x-linux-gnu\" => \"s390x-linux-gnu-\"\n    },\n    {:unix, :darwin} => %{\n      \"x86_64-apple-darwin\" => {\n        \"gcc\",\n        \"g++\",\n        \"<%= cc %> -arch x86_64\",\n        \"<%= cxx %> -arch x86_64\"\n      },\n      \"aarch64-apple-darwin\" => {\n        \"gcc\",\n        \"g++\",\n        \"<%= cc %> -arch arm64\",\n        \"<%= cxx %> -arch arm64\"\n      }\n    },\n    {:win32, :nt} => %{\n      \"x86_64-windows-msvc\" => {\"cl\", \"cl\"}\n    }\n  }\n\n  defp default_compilers, do: @default_compilers\n  defp user_config, do: Mix.Project.config()[:cc_precompile] || default_compilers()\n  defp compilers, do: Access.get(user_config(), :compilers, default_compilers())\n  defp compilers_current_os, do: Access.get(compilers(), :os.type(), %{})\n\n  @impl ElixirMake.Precompiler\n  def current_target do\n    current_target_from_env = current_target_from_env()\n\n    if current_target_from_env do\n      # overwrite current target triplet\n      {:ok, current_target_from_env}\n    else\n      current_target(:os.type())\n    end\n  end\n\n  defp current_target_from_env do\n    arch = System.get_env(\"TARGET_ARCH\")\n    os = System.get_env(\"TARGET_OS\")\n    abi = System.get_env(\"TARGET_ABI\")\n\n    if !Enum.all?([arch, os, abi], &Kernel.is_nil/1) do\n      \"#{arch}-#{os}-#{abi}\"\n    end\n  end\n\n  def current_target({:win32, _}) do\n    processor_architecture =\n      String.downcase(String.trim(System.get_env(\"PROCESSOR_ARCHITECTURE\")))\n\n    # https://docs.microsoft.com/en-gb/windows/win32/winprog64/wow64-implementation-details?redirectedfrom=MSDN\n    partial_triplet =\n      case processor_architecture do\n        \"amd64\" ->\n          \"x86_64-windows-\"\n\n        \"ia64\" ->\n          \"ia64-windows-\"\n\n        \"arm64\" ->\n          \"aarch64-windows-\"\n\n        \"x86\" ->\n          \"x86-windows-\"\n      end\n\n    {compiler, _} = :erlang.system_info(:c_compiler_used)\n\n    case compiler do\n      :msc ->\n        {:ok, partial_triplet <> \"msvc\"}\n\n      :gnuc ->\n        {:ok, partial_triplet <> \"gnu\"}\n\n      other ->\n        {:ok, partial_triplet <> Atom.to_string(other)}\n    end\n  end\n\n  def current_target({:unix, _}) do\n    # get current target triplet from `:erlang.system_info/1`\n    system_architecture = to_string(:erlang.system_info(:system_architecture))\n    current = String.split(system_architecture, \"-\", trim: true)\n\n    case length(current) do\n      4 ->\n        {:ok, \"#{Enum.at(current, 0)}-#{Enum.at(current, 2)}-#{Enum.at(current, 3)}\"}\n\n      3 ->\n        case :os.type() do\n          {:unix, :darwin} ->\n            # could be something like aarch64-apple-darwin21.0.0\n            # but we don't really need the last 21.0.0 part\n            if String.match?(Enum.at(current, 2), ~r/^darwin.*/) do\n              {:ok, \"#{Enum.at(current, 0)}-#{Enum.at(current, 1)}-darwin\"}\n            else\n              {:ok, system_architecture}\n            end\n\n          _ ->\n            {:ok, system_architecture}\n        end\n\n      _ ->\n        {:error, \"cannot decide current target\"}\n    end\n  end\n\n  @impl ElixirMake.Precompiler\n  def all_supported_targets(:compile) do\n    # This callback is expected to return a list of string for\n    # all supported targets by this precompiler. in this\n    # implementation, we will try to find a few crosscompilers\n    # available in the system.\n    #\n    # Note that this implementation is mainly used for demostration\n    # purpose, therefore the hardcoded compiler names are used in\n    # DEBIAN/Ubuntu Linux (as I only installed these ones at the\n    # time of writting this example)\n    with {:ok, current} <- current_target() do\n      Enum.uniq([current] ++ find_all_available_targets())\n    else\n      _ ->\n        []\n    end\n  end\n\n  @impl ElixirMake.Precompiler\n  def all_supported_targets(:fetch) do\n    Enum.flat_map(compilers(), &Map.keys(elem(&1, 1)))\n  end\n\n  defp find_all_available_targets do\n    Map.keys(compilers_current_os())\n    |> Enum.map(&find_available_compilers(&1, Map.get(compilers_current_os(), &1)))\n    |> Enum.reject(&is_nil/1)\n  end\n\n  defp find_available_compilers(triplet, compilers) when is_tuple(compilers) do\n    if System.find_executable(elem(compilers, 0)) do\n      Logger.debug(\"Found compiler for #{triplet}\")\n      triplet\n    else\n      Logger.debug(\"Compiler not found for #{triplet}\")\n      nil\n    end\n  end\n\n  defp find_available_compilers(triplet, invalid) do\n    Mix.raise(\"Invalid configuration for #{triplet}, expecting a 2-tuple or 4-tuple, however, got #{inspect(invalid)}\")\n  end\n\n  @impl ElixirMake.Precompiler\n  def build_native(args) do\n    # In this callback we just build the NIF library natively,\n    # and because this precompiler module is designed for NIF\n    # libraries that use C/C++ as the main language with Makefile,\n    # we can just call `ElixirMake.Precompiler.mix_compile(args)`\n    #\n    # It's also possible to forward this call to:\n    #\n    #     precompile(args, elem(current_target(), 1))\n    #\n    # This could be useful when the precompiler is using a universal\n    # (cross-)compiler, say zig. in this way, the compiled binaries\n    # (`mix compile`) will be consistent as the corrsponding precompiled\n    # one (with `mix elixir_make.precompile`)\n    #\n    # However, if you'd prefer to having the same behaviour for `mix compile`\n    # then the following line is okay\n    ElixirMake.Precompiler.mix_compile(args)\n  end\n\n  @impl ElixirMake.Precompiler\n  def precompile(args, target) do\n    # Potentially clean the output directory to avoid conflicts\n    File.rm!(Path.join(Mix.Project.app_path(), \"priv\"))\n\n    saved_cc = System.get_env(\"CC\") || \"\"\n    saved_cxx = System.get_env(\"CXX\") || \"\"\n    saved_cpp = System.get_env(\"CPP\") || \"\"\n\n    Logger.debug(\"Current compiling target: #{target}\")\n\n    {cc, cxx} = get_cc_and_cxx(target)\n    System.put_env(\"CC\", cc)\n    System.put_env(\"CXX\", cxx)\n    System.put_env(\"CPP\", cxx)\n\n    ElixirMake.Precompiler.mix_compile(args)\n\n    System.put_env(\"CC\", saved_cc)\n    System.put_env(\"CXX\", saved_cxx)\n    System.put_env(\"CPP\", saved_cpp)\n\n    :ok\n  end\n\n  defp get_cc_and_cxx(triplet) do\n    case Access.get(compilers_current_os(), triplet, nil) do\n      nil ->\n        cc = System.get_env(\"CC\")\n        cxx = System.get_env(\"CXX\")\n        cpp = System.get_env(\"CPP\")\n\n        case {cc, cxx, cpp} do\n          {nil, _, _} ->\n            {\"gcc\", \"g++\"}\n\n          {_, nil, nil} ->\n            {\"gcc\", \"g++\"}\n\n          {_, _, nil} ->\n            {cc, cxx}\n\n          {_, nil, _} ->\n            {cc, cpp}\n\n          {_, _, _} ->\n            {cc, cxx}\n        end\n\n      {cc, cxx} ->\n        {cc, cxx}\n\n      prefix when is_binary(prefix) ->\n        {\"#{prefix}gcc\", \"#{prefix}g++\"}\n\n      {cc, cxx, cc_args, cxx_args} ->\n        {\"#{cc} #{cc_args}\", \"#{cxx} #{cxx_args}\"}\n    end\n  end\n\n  @impl ElixirMake.Precompiler\n  def post_precompile_target(target) do\n    # It's possible to do some cleanup work\n    # in this optionall callback\n    # it will be called when `target` is properly archived\n    # so you may safely delete all target-specific files,\n    # like call `make clean`\n    Logger.debug(\"Post target archive\")\n  end\n\n  @impl ElixirMake.Precompiler\n  def post_precompile() do\n    # It's possible to do some post precompilation work\n    # in this optionall callback\n    # after all precompile targets are compiled.\n    Logger.debug(\"Post precompile\")\n  end\nend\n```","ref":"precompilation_guide.html#write-the-cc-precompiler"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.8.2 (2024-03-10) - Changelog","doc":"* Fix checksum error when checksuming NIF version","ref":"changelog.html#v0-8-2-2024-03-10"},{"type":"extras","title":"v0.8.1 (2024-03-10) - Changelog","doc":"* Fix checksum error when falling back to a previous NIF version","ref":"changelog.html#v0-8-1-2024-03-10"},{"type":"extras","title":"v0.8.0 (2024-03-10) - Changelog","doc":"* Fallback to the previous compatible NIF version by default\n  * Deprecate availability in favor of passing a function to `:versions`","ref":"changelog.html#v0-8-0-2024-03-10"},{"type":"extras","title":"v0.7.8 (2024-01-17) - Changelog","doc":"* List certifi as an optional dependency","ref":"changelog.html#v0-7-8-2024-01-17"},{"type":"extras","title":"v0.7.7 (2023-06-01) - Changelog","doc":"* Fix compiler in umbrella apps","ref":"changelog.html#v0-7-7-2023-06-01"},{"type":"extras","title":"v0.7.6 (2023-03-17) - Changelog","doc":"* Do not display error message when a target is unavailable\n  * Allow usage of castore 1.0","ref":"changelog.html#v0-7-6-2023-03-17"},{"type":"extras","title":"v0.7.5 (2023-02-18) - Changelog","doc":"* Support precompiling for multiple NIF versions","ref":"changelog.html#v0-7-5-2023-02-18"},{"type":"extras","title":"v0.7.4 (2023-02-17) - Changelog","doc":"* Preload Erlang/OTP applications for Elixir v1.15+","ref":"changelog.html#v0-7-4-2023-02-17"},{"type":"extras","title":"v0.7.3 (2022-12-27) - Changelog","doc":"* Add `post_precompile_target` to `ElixirMake.Precompiler` behaviour","ref":"changelog.html#v0-7-3-2022-12-27"},{"type":"extras","title":"v0.7.2 (2022-12-14) - Changelog","doc":"* Allow precompiler to configure behaviour for unavailable targets","ref":"changelog.html#v0-7-2-2022-12-14"},{"type":"extras","title":"v0.7.1 (2022-12-07) - Changelog","doc":"* Use CACerts from Erlang/OTP 25 if available","ref":"changelog.html#v0-7-1-2022-12-07"},{"type":"extras","title":"v0.7.0 (2022-12-02) - Changelog","doc":"* Support precompilation with custom precompilers\n  * Don't pass default Erlang environment variables into make","ref":"changelog.html#v0-7-0-2022-12-02"},{"type":"extras","title":"v0.6.3 (2021-10-19) - Changelog","doc":"* Fallback to `make` if `nmake` is not available on Windows.","ref":"changelog.html#v0-6-3-2021-10-19"},{"type":"extras","title":"v0.6.2 (2020-12-03) - Changelog","doc":"* Fix permissions for some files in the repository.","ref":"changelog.html#v0-6-2-2020-12-03"},{"type":"extras","title":"v0.6.1 (2020-09-07) - Changelog","doc":"* Warn on paths that contain spaces.\n  * Use `gmake` on NetBSD.","ref":"changelog.html#v0-6-1-2020-09-07"},{"type":"extras","title":"v0.6.0 (2019-06-10) - Changelog","doc":"* Start tracking CHANGELOG.","ref":"changelog.html#v0-6-0-2019-06-10"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}